@@ -1,26 +1,46 @@
-import os
+import sys
 import errno
 import hashlib
+import gzip
+import os
+from requests import Session, head, RequestException
 from re import findall
 from time import time
 from traceback import print_exc
-from requests import get, head
+from gitlab_ps_utils.decorators import stable_retry
 
 
+@stable_retry(retries=5, delay=60, backoff=1.2)
 def download_file(url, path, filename=None, headers=None, verify=True):
-    # NOTE the stream=True parameter
+    '''
+    Uses the stable_retry decorator to attempt to stream download a file, typically
+    a GitLab .tar.gz file. Any requests status other than 200 will result in
+    an error, prompting the stable_retry.
+    '''
     if __is_downloadable(url, verify):
-        r = get(url, stream=True, headers=headers,
-                allow_redirects=True, verify=verify)
-        if filename is None:
-            filename = __get_filename_from_cd(
-                r.headers.get('content-disposition'))
-        file_path = "{0}/downloads/{1}".format(path, filename)
-        create_local_project_export_structure(os.path.dirname(file_path))
-        with open(file_path, "wb") as f:
-            for chunk in r.iter_content(chunk_size=1024):
-                if chunk:
-                    f.write(chunk)
+        chunk_size = 1024*1024
+        session = Session()
+        try:
+            r = session.get(url, stream=True, headers=headers,
+                            allow_redirects=True, verify=verify)
+            r.raise_for_status()
+            if filename is None:
+                filename = __get_filename_from_cd(
+                    r.headers.get('content-disposition'))
+            file_path = f"{path}/downloads/{filename}"
+            create_local_project_export_structure(os.path.dirname(file_path))
+            with open(file_path, "wb", chunk_size) as f:
+                for chunk in r.iter_content(chunk_size=chunk_size):
+                    if chunk:
+                        f.write(chunk)
+        except RequestException as re:
+            print(
+                f"Download request to {url} failed for {filename} ({path}) due to:\n{re}", file=sys.stderr)
+            return None
+        except Exception as e:
+            print(
+                f"Failed to download {filename} ({path}) from  {url} due to:\n{e}", file=sys.stderr)
+            return None
     return filename
 
 
@@ -33,6 +53,20 @@ def create_local_project_export_structure(dir_path):
                 raise
 
 
+def is_gzip(test_file):
+    '''
+    Attempts to detect is a file is a valid gzip file.  This assists with detecting
+    accuracy of the export.  Even if the file is detected as a GZIP, its possible the
+    download still failed in some other way, and is incomplete.
+    '''
+    with gzip.open(test_file, 'r') as fh:
+        try:
+            fh.read(1)
+            return True
+        except gzip.BadGzipFile:
+            return False
+
+
 def __is_downloadable(url, verify):
     """
         Does the url contain a downloadable resource
@@ -63,8 +97,10 @@ def is_recent_file(path, age=2592000):
     """
         Check whether a file path exists, is empty and older than 1 month
     """
-    return os.path.exists(path) and os.path.getsize(
-        path) > 0 and (time() - os.path.getmtime(path) < age)
+    try:
+        return os.path.exists(path) and os.path.getsize(path) > 0 and (time() - os.path.getmtime(path) < age)
+    except OSError as ose:
+        sys.exit(f"Path {path} does not exist or is inaccessible: {ose}")
 
 
 def get_hash_of_dirs(directory, verbose=0):
